module Calendar.FrenchRevolutionary where

import Calendar.Global
import Calendar.Gregorian 

data FrenchRevolutionary = FrenchRevolutionaryData {frenchrev'year::Fixed,
                                                    frenchrev'month::Fixed,
                                                    frenchrev'day::Fixed} 
                         deriving (Eq)

frenchRevEpoch = fixedFromGregorian (GregorianData 1792 9 22)

instance Show FrenchRevolutionary where
    show = \r -> 
           show (frenchrev'year r) ++ "-" ++
           show (frenchrev'month r) ++ "-" ++
           show (frenchrev'day r)

prettyShow r =
    show (frenchrev'day r) ++ " " ++
    case(frenchrev'month r) of
               1 -> "Vendémiaire"
               2 -> "Brumaire"      
               3 -> "Frimaire"
               4 -> "Nivôse"
               5 -> "Pluviöse"
               6 -> "Ventôse"
               7 -> "Germinal"
               8 -> "Floréal"
               9 -> "Prairial"
               10 -> "Messidor"
               11 -> "Thermidor"
               12 -> "Fructidor"
               otherwise -> error "Month out of range"
           ++ " " ++
     show (frenchrev'year r) 

-- The original form of the calendar is astronomical. There is no rule
-- for leap years, for instance. We implement here only the modified
-- form proposed by Gilbert Romme (which was never deployed)

modifiedFrenchRevLeapYear y =
    y `mod` 4 == 0 &&
      (y `mod` 400 == 100 ||
       y `mod` 400 == 200 ||
       y `mod` 400 == 300) &&
    y `mod` 4000 /= 0

fixedFromModifiedFrenchRev r =
    let year = frenchrev'year r in
    frenchRevEpoch - 1 + 365 * (year - 1) +
           floor (fromIntegral (year - 1) / 4) -
           floor (fromIntegral (year - 1) / 100) +
           floor (fromIntegral (year - 1) / 400) -
           floor (fromIntegral (year - 1) / 4000) +
           30 * (frenchrev'month r - 1) + frenchrev'day r

modifiedFrenchRevFromFixed f =
    let approx = floor (fromIntegral (f - frenchRevEpoch + 2) / 
                                         (1460969 / 4000)) + 1 in
    let year = if f < fixedFromModifiedFrenchRev 
                          (FrenchRevolutionaryData approx 1 1) then
                   approx - 1
               else
                   approx 
         in
    let month = floor (fromIntegral (f - 
                          fixedFromModifiedFrenchRev  
                                 (FrenchRevolutionaryData year 1 1)) / 30) + 1 in
    let day = f - fixedFromModifiedFrenchRev 
                         (FrenchRevolutionaryData year month 1) + 1 in
    FrenchRevolutionaryData year month day
