module Calendar.Islamic where

import Calendar.Global
import Calendar.Julian -- for the definition of the epoch

islamicEpoch = fixedFromJulian (JulianData 622 7 16)

data Islamic = IslamicData {islamic'year::Fixed, islamic'month::Fixed,
                                             islamic'day::Fixed} deriving (Eq)

-- We do not derive Show because ours is better
instance Show Islamic where
    show = \i -> 
           show (islamic'year i) ++ "-" ++
           show (islamic'month i) ++ "-" ++
           show (islamic'day i)

prettyShow i =
           case(islamic'month i) of
               1 -> "Muharram"
               2 -> "Safar"      
               3 -> "Rabi I"
               4 -> "Rabi II"
               5 -> "Jumada I"
               6 -> "Jumada II"
               7 -> "Rajab"
               8 -> "Shaban"
               9 -> "Ramadan"
               10 -> "Shawwal"
               11 -> "Dhu al-Qada"
               12 -> "Dhu al-Hijja"
               otherwise -> error "Month out of range"
           ++ " " ++
           show (islamic'day i) ++ ", " ++
           show (islamic'year i) 

-- Default Eq is fine, we do not need to define it

islamicLeapYear y =
    ((14 + 11 * y) `mod` 30) < 11

fixedFromIslamic id = 
    let year = islamic'year id in
    let month = islamic'month id in
    let day = islamic'day id in
    day + 29 * (month - 1) + floor (fromIntegral (6*month - 1) / 11) + 
            (year -1) * 354 + floor (fromIntegral (3 + 11*year) / 30) + 
                      islamicEpoch - 1

islamicFromFixed fd =
    let year = floor (fromIntegral (30*(fd - islamicEpoch) + 10646) / 10631) in
    let priorDays = fd - fixedFromIslamic (IslamicData year 1 1) in
    let month = floor (fromIntegral(11*priorDays + 330) / 325) in
    let day = fd - fixedFromIslamic(IslamicData year month 1) + 1 in
    IslamicData year month day



