module Calendar.Julian where

import Calendar.Global
import Calendar.Gregorian -- for the definition of epoch

data Julian = JulianData {julian'year::Fixed, julian'month::Fixed,
                                             julian'day::Fixed} deriving (Eq)

instance Show Julian where
    show = \j -> show (julian'year j) ++ "-" ++
           show (julian'month j) ++ "-" ++
           show (julian'day j) 

julianEpoch = fixedFromGregorian (GregorianData 0 12 30)

julianLeapYear y = 
    let modulo = y `mod` 4 in
    if y > 0 then
        modulo == 0
    else
        modulo == 3

fixedFromJulian :: Julian -> Fixed
fixedFromJulian j = 
    let year = julian'year j in
    let month = julian'month j in
    let day = julian'day j in
    let monthOffset =
            if month <= 2 then
                0
            else if julianLeapYear year then
                     -1
                 else
                     -2 in
    let y = if year < 0 then
                year + 1
            else
                year in
    julianEpoch - 1 + 365 * (y - 1) + floor (fromIntegral (y - 1) / 4) +
                floor (fromIntegral (367 * month - 362) / 12) + monthOffset + 
                day

julianFromFixed :: Fixed -> Julian
julianFromFixed f = 
    let approx = floor (fromIntegral (4 * (f - julianEpoch) + 1464) / 1461) in
    let year = if approx <= 0 then
                   approx - 1
               else
                   approx in
    let priorDays = f - fixedFromJulian (JulianData year 1 1) in
    let correction = if f < fixedFromJulian (JulianData year 3 1) then
                         0
                         else if julianLeapYear year then
                                  1
                              else
                                  2 in
    let month = floor (fromIntegral (12 * (priorDays + correction) + 373) / 367) in
    let day = f - fixedFromJulian (JulianData year month 1) + 1 in
    JulianData year month day
            
