module Calendar.Mayan where

import Calendar.Global

-- LongCount is the one for long durations and far away dates
data MayanLongCount = MayanLongCountData {baktun::Fixed, katun::Fixed, 
                                          tun::Fixed, uinal::Fixed, kin::Fixed}
                    deriving (Eq, Show)

-- Haab is the practical calendar, for ordinary people
data MayanHaab = MayanHaabData {haab'month::Fixed, haab'day::Fixed} 
                 deriving (Eq, Show)

-- Tzolkin is a religious calendar
data MayanTzolkin = MayanTzolkinData {tzolkin'number::Fixed, tzolkin'name::Fixed} deriving (Show)

prettyShow m =
    show (baktun m) ++ "." ++
    show (katun m) ++ "." ++
    show (tun m) ++ "." ++
    show (uinal m) ++ "." ++
    show (kin m)

prettyShowHaab m =
    show (haab'day m) ++ " " ++
    case(haab'month m) of
               1 -> "Pop"
               2 -> "Uo"      
               3 -> "Zip"
               4 -> "Zotz"
               5 -> "Tzec"
               6 -> "Xul"
               7 -> "Yaxkin"
               8 -> "Mol"
               9 -> "Chen"
               10 -> "Yax"
               11 -> "Zac"
               12 -> "Ceh"
               13 -> "Mac"
               14 -> "Kankin"
               15 -> "Muan"
               16 -> "Pax"
               17 -> "Kayab"
               18 -> "Cumku"
               19 -> "Uayeb"
               otherwise -> error "Month out of range"

prettyShowTzolkin m =
    show (tzolkin'number m) ++ " " ++
    case(tzolkin'name m) of
               1 -> "Imix"
               2 -> "Ik"      
               3 -> "Akbal"
               4 -> "Kan"
               5 -> "Chicchan"
               6 -> "Cimi"
               7 -> "Manik"
               8 -> "Lamat"
               9 -> "Muluc"
               10 -> "Oc"
               11 -> "Chuen"
               12 -> "Eb"
               13 -> "Ben"
               14 -> "Ix"
               15 -> "Men"
               16 -> "Cib"
               17 -> "Caban"
               18 -> "Eznab"
               19 -> "Cauac"
               20 -> "Ahau"
               otherwise -> error "Month out of range"

amod x y = 
    let modulo = x `mod` y in
    if modulo == 0 then
        y
    else
        modulo

mayanEpoch = fixedFromJD 584283

fixedFromMayanLongCount m =
    mayanEpoch + (baktun m) * 144000 + (katun m) * 7200 +
               (tun m) * 360 + (uinal m) * 20 + (kin m)

mayanLongCountFromFixed f =
    let longCount = f - mayanEpoch in
    let baktun = floor (fromIntegral longCount / 144000) in
    let dayOfBaktun = longCount `mod` 144000 in
    let katun = floor (fromIntegral dayOfBaktun / 7200) in
    let dayOfKatun = dayOfBaktun `mod` 7200 in
    let tun = floor (fromIntegral dayOfKatun / 360) in
    let dayOfTun = dayOfKatun `mod` 360 in
    let uinal = floor (fromIntegral dayOfTun / 20) in
    let kin = dayOfTun `mod` 20 in
    MayanLongCountData baktun katun tun uinal kin

mayanHaabOrdinal h =
    (haab'month h - 1) * 20 + haab'day h

mayanHaabEpoch = mayanEpoch - mayanHaabOrdinal (MayanHaabData 18 8)

mayanHaabFromFixed f =
    let count = (f - mayanHaabEpoch) `mod` 365 in
    let day = count `mod` 20 in
    let month = floor (fromIntegral count / 20) + 1 in
    MayanHaabData month day

-- No fixedFromMayanHaab since Haab dates have no year
mayanHaabOnOrBefore h f =
   f - ((f - mayanHaabEpoch - mayanHaabOrdinal(h)) `mod` 365)

mayanTzolkinEpoch =
    mayanEpoch - mayanTzolkinOrdinal (MayanTzolkinData 4 20)

mayanTzolkinOrdinal t =
    (tzolkin'number t) - 1 + 39 * (tzolkin'number t - tzolkin'name t) `mod` 260

mayanTzolkinFromFixed f =
    let count = f - mayanTzolkinEpoch + 1 in
    let number = count `amod` 13 in
    let name = count `amod` 20 in
    MayanTzolkinData number name

mayanTzolkinOnOrBefore t f =
    f - ((f - mayanTzolkinEpoch - mayanTzolkinOrdinal (t)) `mod` 260)
