import Test.HUnit
import Calendar.Global
import Calendar.Egyptian
import Calendar.Julian
import Calendar.Gregorian
import Calendar.Islamic
import Calendar.Mayan
import Calendar.FrenchRevolutionary

firstEgyptian = egyptianFromFixed (egyptianEpoch)

testsEgyptian = TestList [
                  TestCase (assertBool "Epoch" 
                                       ((egyptian'year firstEgyptian == 1) && 
                                       (egyptian'month firstEgyptian == 1)))
                   ]

firstJulian = julianFromFixed (julianEpoch)

testsJulian = TestList [
                  TestCase (assertBool "Epoch" 
                                       ((julian'year firstJulian == 1) && 
                                       (julian'month firstJulian == 1))),
                  TestCase (assertBool "Negative date" 
                                       (julian'year 
                                              (julianFromFixed (-2)) == 
                                                      -1)), -- No year 0
                  TestCase (assertBool "Hegira" 
                                       (fixedFromJulian (JulianData 622 7 16) == 227015))
                        ]

testsGregorian = TestList [
                  TestCase (assertBool "Millenium edition" 
                                       (gregorianFromFixed 730120 ==
                                                          GregorianData 2000 1 1)),
                  TestCase (assertBool "Today"
                                       (fixedFromGregorian (GregorianData 
                                                            2008 1 30) == 733071)),
                  TestCase (assertBool "Birth of JC" 
                                       (gregorianFromFixed gregorianEpoch ==
                                        GregorianData 1 1 1)),
                  TestCase (assertBool "One day offset"
                                       (fixedFromGregorian (GregorianData 1613 11 26) -
                                       fixedFromGregorian (GregorianData 1613 11 25) == 1)),
                  TestCase (assertBool "One day offset over a month"
                                       (fixedFromGregorian (GregorianData 2453 9 1) -
                                       fixedFromGregorian (GregorianData 2453 8 31) == 1)),
                  TestCase (assertBool "One day offset over a year"
                                       (fixedFromGregorian (GregorianData 4533 1 1) -
                                       fixedFromGregorian (GregorianData 4532 12 31) == 1)),
                  TestCase (assertBool "Negative date" 
                                       (gregorian'year 
                                              (gregorianFromFixed (-1)) == -- This assumes the epoch is 1...
                                                      0)), -- Unlike the Julian calendar, the Gregorian does have a year 0
                  TestCase (assertBool "Negative date, further" 
                                       (gregorian'year 
                                              (gregorianFromFixed (-1000)) ==
                                                      -2)),
                  TestCase (assertBool "Leap year"
                                       (fixedFromGregorian (GregorianData 1960 3 1) -
                                       fixedFromGregorian (GregorianData 1960 2 28) == 2)),
                  TestCase (assertBool "Gregorian special leap year"
                                       (fixedFromGregorian (GregorianData 1900 3 1) -
                                       fixedFromGregorian (GregorianData 1900 2 28) == 1)),
                  TestCase (assertBool "Gregorian very special leap year"
                                       (fixedFromGregorian (GregorianData 3200 3 1) -
                                       fixedFromGregorian (GregorianData 3200 2 28) == 2))
                 ]

testsIslamic = TestList [
                  TestCase (assertBool "Today"
                                       (fixedFromIslamic (IslamicData 
                                                            1429 1 21) == 733071)),
                  TestCase (assertBool "Go to Medina" 
                                       (islamicFromFixed islamicEpoch ==
                                        IslamicData 1 1 1)),
                  TestCase (assertBool "One day offset"
                                       (fixedFromIslamic (IslamicData 1613 11 26) -
                                       fixedFromIslamic (IslamicData 1613 11 25) == 1)),
                  TestCase (assertBool "One day offset over a month"
                                       (fixedFromIslamic (IslamicData 2453 8 1) -
                                       fixedFromIslamic (IslamicData 2453 7 30) == 1)),
                  TestCase (assertBool "One day offset over a year" -- 4532 is leap year
                                       (fixedFromIslamic (IslamicData 4533 1 1) -
                                       fixedFromIslamic (IslamicData 4532 12 30) == 1)),
                  TestCase (assertBool "Negative date" 
                                       (islamic'year 
                                              (islamicFromFixed (islamicEpoch -1)) ==
                                                      0)), -- Unlike the Julian 
                  -- calendar, the Islamic does have a year 0? Not everyone agrees 
                  -- (and muslim calendars seldom manage pre-islamic dates)
                  TestCase (assertBool "Negative date, further" 
                                       (islamic'year 
                                              (islamicFromFixed (islamicEpoch -1000)) ==
                                                      -2)),
                  TestCase (assertBool "Non leap year 1429"
                                       (fixedFromIslamic (IslamicData 1430 1 1) -
                                       fixedFromIslamic (IslamicData 1429 12 29) == 1))
                 ]

testsMayan = testsMayanLongCount

testsMayanLongCount = TestList [
                       TestCase (assertBool "An ordinary date"
                                          (fixedFromMayanLongCount 
                                           (MayanLongCountData 12 19 15 0 18) == 733076)),
                       TestCase (assertBool "Epoch" 
                                           ( mayanLongCountFromFixed mayanEpoch ==
                                                                     MayanLongCountData 0 0 0 0 0))]

testsFrenchRevolutionary = TestList [
                              TestCase (assertBool "An ordinary date"
                                                   (fixedFromModifiedFrenchRev 
                                                    (FrenchRevolutionaryData 216 5 16) == 733076)),
                              TestCase (assertBool "Epoch" 
                                                   ((modifiedFrenchRevFromFixed 
                                                    frenchRevEpoch) ==
                                                    (FrenchRevolutionaryData 1 1 1)))
                             ]

--                   -- CC, appendix C
--                  TestCase (assertBool "-214193"
--                                       (egyptianFromFixed (-214193) ==
--                                                          (EgyptianData 161 7 15)))
buildTestCase fixed jd egyptian gregorian islamic mayan =
    [
      TestCase (assertBool ("JD (julian days) " ++ show fixed) (
                 (jdFromFixed fixed == jd))),
      TestCase (assertBool ("Egyptian " ++ show fixed) (
                 (egyptianFromFixed fixed == egyptian))),
      TestCase (assertBool ("Gregorian " ++ show fixed) (
                 (gregorianFromFixed fixed == gregorian))),
      TestCase (assertBool ("Islamic " ++ show fixed) (           
                (islamicFromFixed fixed == islamic))),
       TestCase (assertBool ("Mayan Long Count " ++ show fixed) (          
                (mayanLongCountFromFixed fixed == mayan)))
    ]

testsAppendixC = TestList 
                   ( (buildTestCase (-214193) 1507231.5 (EgyptianData 161 7 15) 
                                  (GregorianData (-586) 7 24) (IslamicData (-1245) 12 9)
                                  (MayanLongCountData 6 8 3 13 9)) ++
                     (buildTestCase (25469) 1746893.5 (EgyptianData 818 2 22)
                                  (GregorianData 70 9 24) (IslamicData (-568)  4 1)
                                  (MayanLongCountData 8 1 9 8 11)) ++                                     (buildTestCase (708842) 2430266.5 (EgyptianData 2690 5 25)
                                  (GregorianData 1941 9 29) (IslamicData 1360 9 8)
                                  (MayanLongCountData 12 16 7 13 4)) ++
                     (buildTestCase (744313) 2465737.5 (EgyptianData 2787 8 1)
                                  (GregorianData 2038 11 10) (IslamicData 1460 10 12)
                                  (MayanLongCountData 13 1 6 4 15)) ++
                     (buildTestCase (764652) 2486076.5 (EgyptianData 2843 4 20)
                                  (GregorianData 2094 7 18) (IslamicData 1518 3 5)
                                  (MayanLongCountData 13 4 2 13 14))
                     )

main = do
  putStrLn "Egyptian"
  runTestTT testsEgyptian
  putStrLn "Julian"
  runTestTT testsJulian
  putStrLn "Gregorian"
  runTestTT testsGregorian
  putStrLn "Islamic"
  runTestTT testsIslamic
  putStrLn "Mayan"
  runTestTT testsMayan
  putStrLn "French revolutionary"
  runTestTT testsFrenchRevolutionary
  putStrLn "Appendix C tests"
  runTestTT testsAppendixC
